  CREATE TABLE USER1.TABLE1
   (	FIELD_ID NUMBER(19,0) NOT NULL ENABLE,
	OTHERFIELD NUMBER(19,0) NOT NULL ENABLE,
	 CONSTRAINT FK_OTHERFIELD FOREIGN KEY (OTHERFIELD)
	  REFERENCES USER1.TABLE2 (ID) ON DELETE CASCADE ENABLE,
	 CONSTRAINT FK_FIELD_ID FOREIGN KEY (FIELD_ID)
	  REFERENCES USER1.TABLE3 (ID) ON DELETE CASCADE ENABLE
   ) ;
